@@ -1,7 +1,7 @@
 /*
  * Copyright (C) 2010-2011 Robert Ancell.
  * Author: Robert Ancell <robert.ancell@canonical.com>
- * 
+ *
  * This program is free software: you can redistribute it and/or modify it under
  * the terms of the GNU General Public License as published by the Free Software
  * Foundation, either version 3 of the License, or (at your option) any later
@@ -20,7 +20,6 @@
 #include "seat-xlocal.h"
 #include "seat-xremote.h"
 #include "seat-unity.h"
-#include "seat-surfaceflinger.h"
 #include "plymouth.h"
 
 enum {
@@ -112,7 +111,7 @@ display_manager_add_seat (DisplayManager *manager, Seat *seat)
         return FALSE;
 
     manager->priv->seats = g_list_append (manager->priv->seats, g_object_ref (seat));
-    g_signal_connect (seat, "stopped", G_CALLBACK (seat_stopped_cb), manager);
+    g_signal_connect (seat, SEAT_SIGNAL_STOPPED, G_CALLBACK (seat_stopped_cb), manager);
     g_signal_emit (manager, signals[SEAT_ADDED], 0, seat);
 
     return TRUE;
@@ -126,7 +125,7 @@ display_manager_start (DisplayManager *manager)
     /* Disable Plymouth if no X servers are replacing it */
     if (plymouth_get_is_active ())
     {
-        g_debug ("Stopping Plymouth, no displays replace it");      
+        g_debug ("Stopping Plymouth, no displays replace it");
         plymouth_quit (FALSE);
     }
 }
@@ -156,7 +155,7 @@ display_manager_stop (DisplayManager *manager)
 
     check_stopped (manager);
 }
-  
+
 static void
 display_manager_init (DisplayManager *manager)
 {
@@ -166,7 +165,6 @@ display_manager_init (DisplayManager *manager)
     seat_register_module ("xlocal", SEAT_XLOCAL_TYPE);
     seat_register_module ("xremote", SEAT_XREMOTE_TYPE);
     seat_register_module ("unity", SEAT_UNITY_TYPE);
-    seat_register_module ("surfaceflinger", SEAT_SURFACEFLINGER_TYPE);
 }
 
 static void
@@ -197,7 +195,7 @@ display_manager_class_init (DisplayManagerClass *klass)
     g_type_class_add_private (klass, sizeof (DisplayManagerPrivate));
 
     signals[SEAT_ADDED] =
-        g_signal_new ("seat-added",
+        g_signal_new (DISPLAY_MANAGER_SIGNAL_SEAT_ADDED,
                       G_TYPE_FROM_CLASS (klass),
                       G_SIGNAL_RUN_LAST,
                       G_STRUCT_OFFSET (DisplayManagerClass, seat_added),
@@ -205,7 +203,7 @@ display_manager_class_init (DisplayManagerClass *klass)
                       NULL,
                       G_TYPE_NONE, 1, SEAT_TYPE);
     signals[SEAT_REMOVED] =
-        g_signal_new ("seat-removed",
+        g_signal_new (DISPLAY_MANAGER_SIGNAL_SEAT_REMOVED,
                       G_TYPE_FROM_CLASS (klass),
                       G_SIGNAL_RUN_LAST,
                       G_STRUCT_OFFSET (DisplayManagerClass, seat_removed),
@@ -213,7 +211,7 @@ display_manager_class_init (DisplayManagerClass *klass)
                       NULL,
                       G_TYPE_NONE, 1, SEAT_TYPE);
     signals[STOPPED] =
-        g_signal_new ("stopped",
+        g_signal_new (DISPLAY_MANAGER_SIGNAL_STOPPED,
                       G_TYPE_FROM_CLASS (klass),
                       G_SIGNAL_RUN_LAST,
                       G_STRUCT_OFFSET (DisplayManagerClass, stopped),
