@@ -1,7 +1,7 @@
 /*
  * Copyright (C) 2010-2011 Robert Ancell.
  * Author: Robert Ancell <robert.ancell@canonical.com>
- * 
+ *
  * This program is free software: you can redistribute it and/or modify it under
  * the terms of the GNU General Public License as published by the Free Software
  * Foundation, either version 3 of the License, or (at your option) any later
@@ -44,11 +44,11 @@ struct GreeterPrivate
     guint8 *read_buffer;
     gsize n_read;
     gboolean use_secure_memory;
-  
+
     /* Hints for the greeter */
     GHashTable *hints;
 
-    /* Default session to use */   
+    /* Default session to use */
     gchar *default_session;
 
     /* Sequence number of current PAM session */
@@ -63,6 +63,9 @@ struct GreeterPrivate
     /* PAM session being constructed by the greeter */
     Session *authentication_session;
 
+    /* TRUE if a the greeter can handle a reset; else we will just kill it instead */
+    gboolean resettable;
+
     /* TRUE if a user has been authenticated and the session requested to start */
     gboolean start_session;
 
@@ -104,6 +107,8 @@ typedef enum
     SERVER_MESSAGE_END_AUTHENTICATION,
     SERVER_MESSAGE_SESSION_RESULT,
     SERVER_MESSAGE_SHARED_DIR_RESULT,
+    SERVER_MESSAGE_IDLE,
+    SERVER_MESSAGE_RESET,
 } ServerMessage;
 
 static gboolean read_cb (GIOChannel *source, GIOCondition condition, gpointer data);
@@ -129,6 +134,12 @@ greeter_set_allow_guest (Greeter *greeter, gboolean allow_guest)
     greeter->priv->allow_guest = allow_guest;
 }
 
+void
+greeter_clear_hints (Greeter *greeter)
+{
+    g_hash_table_remove_all (greeter->priv->hints);
+}
+
 void
 greeter_set_hint (Greeter *greeter, const gchar *name, const gchar *value)
 {
@@ -217,7 +228,7 @@ static void
 write_string (guint8 *buffer, gint buffer_length, const gchar *value, gsize *offset)
 {
     gint length;
-  
+
     if (value)
         length = strlen (value);
     else
@@ -249,7 +260,7 @@ string_length (const gchar *value)
 }
 
 static void
-handle_connect (Greeter *greeter, const gchar *version)
+handle_connect (Greeter *greeter, const gchar *version, gboolean resettable)
 {
     guint8 message[MAX_MESSAGE_LENGTH];
     gsize offset = 0;
@@ -257,7 +268,9 @@ handle_connect (Greeter *greeter, const gchar *version)
     GHashTableIter iter;
     gpointer key, value;
 
-    l_debug (greeter, "Greeter connected version=%s", version);
+    l_debug (greeter, "Greeter connected version=%s resettable=%s", version, resettable ? "true" : "false");
+
+    greeter->priv->resettable = resettable;
 
     length = string_length (VERSION);
     g_hash_table_iter_init (&iter, greeter->priv->hints);
@@ -296,7 +309,7 @@ pam_messages_cb (Session *session, Greeter *greeter)
     size = int_length () + string_length (session_get_username (session)) + int_length ();
     for (i = 0; i < messages_length; i++)
         size += int_length () + string_length (messages[i].msg);
-  
+
     write_header (message, MAX_MESSAGE_LENGTH, SERVER_MESSAGE_PROMPT_AUTHENTICATION, size, &offset);
     write_int (message, MAX_MESSAGE_LENGTH, greeter->priv->authentication_sequence_number, &offset);
     write_string (message, MAX_MESSAGE_LENGTH, session_get_username (session), &offset);
@@ -332,7 +345,40 @@ send_end_authentication (Greeter *greeter, guint32 sequence_number, const gchar
     write_int (message, MAX_MESSAGE_LENGTH, sequence_number, &offset);
     write_string (message, MAX_MESSAGE_LENGTH, username, &offset);
     write_int (message, MAX_MESSAGE_LENGTH, result, &offset);
-    write_message (greeter, message, offset); 
+    write_message (greeter, message, offset);
+}
+
+void
+greeter_idle (Greeter *greeter)
+{
+    guint8 message[MAX_MESSAGE_LENGTH];
+    gsize offset = 0;
+
+    write_header (message, MAX_MESSAGE_LENGTH, SERVER_MESSAGE_IDLE, 0, &offset);
+    write_message (greeter, message, offset);
+}
+
+void
+greeter_reset (Greeter *greeter)
+{
+    guint8 message[MAX_MESSAGE_LENGTH];
+    gsize offset = 0;
+    guint32 length = 0;
+    GHashTableIter iter;
+    gpointer key, value;
+
+    g_hash_table_iter_init (&iter, greeter->priv->hints);
+    while (g_hash_table_iter_next (&iter, &key, &value))
+        length += string_length (key) + string_length (value);
+
+    write_header (message, MAX_MESSAGE_LENGTH, SERVER_MESSAGE_RESET, length, &offset);
+    g_hash_table_iter_init (&iter, greeter->priv->hints);
+    while (g_hash_table_iter_next (&iter, &key, &value))
+    {
+        write_string (message, MAX_MESSAGE_LENGTH, key, &offset);
+        write_string (message, MAX_MESSAGE_LENGTH, value, &offset);
+    }
+    write_message (greeter, message, offset);
 }
 
 static void
@@ -392,7 +438,7 @@ handle_login (Greeter *greeter, guint32 sequence_number, const gchar *username)
     if (greeter->priv->active_username)
         g_free (greeter->priv->active_username);
     greeter->priv->active_username = g_strdup (username);
-    g_object_notify (G_OBJECT (greeter), "active-username");
+    g_object_notify (G_OBJECT (greeter), GREETER_PROPERTY_ACTIVE_USERNAME);
 
     greeter->priv->authentication_sequence_number = sequence_number;
     g_signal_emit (greeter, signals[CREATE_SESSION], 0, &greeter->priv->authentication_session);
@@ -402,8 +448,8 @@ handle_login (Greeter *greeter, guint32 sequence_number, const gchar *username)
         return;
     }
 
-    g_signal_connect (G_OBJECT (greeter->priv->authentication_session), "got-messages", G_CALLBACK (pam_messages_cb), greeter);
-    g_signal_connect (G_OBJECT (greeter->priv->authentication_session), "authentication-complete", G_CALLBACK (authentication_complete_cb), greeter);
+    g_signal_connect (G_OBJECT (greeter->priv->authentication_session), SESSION_SIGNAL_GOT_MESSAGES, G_CALLBACK (pam_messages_cb), greeter);
+    g_signal_connect (G_OBJECT (greeter->priv->authentication_session), SESSION_SIGNAL_AUTHENTICATION_COMPLETE, G_CALLBACK (authentication_complete_cb), greeter);
 
     /* Use non-interactive service for autologin user */
     autologin_username = g_hash_table_lookup (greeter->priv->hints, "autologin-user");
@@ -440,7 +486,7 @@ handle_login_as_guest (Greeter *greeter, guint32 sequence_number)
         return;
     }
 
-    greeter->priv->guest_account_authenticated = TRUE;  
+    greeter->priv->guest_account_authenticated = TRUE;
     send_end_authentication (greeter, sequence_number, "", PAM_SUCCESS);
 }
 
@@ -506,8 +552,8 @@ handle_login_remote (Greeter *greeter, const gchar *session_name, const gchar *u
     g_signal_emit (greeter, signals[CREATE_SESSION], 0, &greeter->priv->authentication_session);
     if (greeter->priv->authentication_session)
     {
-        g_signal_connect (G_OBJECT (greeter->priv->authentication_session), "got-messages", G_CALLBACK (pam_messages_cb), greeter);
-        g_signal_connect (G_OBJECT (greeter->priv->authentication_session), "authentication-complete", G_CALLBACK (authentication_complete_cb), greeter);
+        g_signal_connect (G_OBJECT (greeter->priv->authentication_session), SESSION_SIGNAL_GOT_MESSAGES, G_CALLBACK (pam_messages_cb), greeter);
+        g_signal_connect (G_OBJECT (greeter->priv->authentication_session), SESSION_SIGNAL_AUTHENTICATION_COMPLETE, G_CALLBACK (authentication_complete_cb), greeter);
 
         /* Run the session process */
         session_set_pam_service (greeter->priv->authentication_session, service);
@@ -744,10 +790,11 @@ read_cb (GIOChannel *source, GIOCondition condition, gpointer data)
     Greeter *greeter = data;
     gsize n_to_read, n_read, offset;
     GIOStatus status;
-    int id, i;
+    int id, length, i;
     guint32 sequence_number, n_secrets, max_secrets;
     gchar *version, *username, *session_name, *language;
     gchar **secrets;
+    gboolean resettable = FALSE;
     GError *error = NULL;
 
     if (condition == G_IO_HUP)
@@ -756,7 +803,7 @@ read_cb (GIOChannel *source, GIOCondition condition, gpointer data)
         greeter->priv->from_greeter_watch = 0;
         return FALSE;
     }
-  
+
     n_to_read = HEADER_SIZE;
     if (greeter->priv->n_read >= HEADER_SIZE)
     {
@@ -792,17 +839,19 @@ read_cb (GIOChannel *source, GIOCondition condition, gpointer data)
             greeter->priv->read_buffer = secure_realloc (greeter, greeter->priv->read_buffer, n_to_read);
             read_cb (source, condition, greeter);
             return TRUE;
-        }      
+        }
     }
-  
+
     offset = 0;
     id = read_int (greeter, &offset);
-    read_int (greeter, &offset);
+    length = HEADER_SIZE + read_int (greeter, &offset);
     switch (id)
     {
     case GREETER_MESSAGE_CONNECT:
         version = read_string (greeter, &offset);
-        handle_connect (greeter, version);
+        if (offset < length)
+            resettable = read_int (greeter, &offset) != 0;
+        handle_connect (greeter, version, resettable);
         g_free (version);
         break;
     case GREETER_MESSAGE_AUTHENTICATE:
@@ -881,6 +930,13 @@ greeter_get_authentication_session (Greeter *greeter)
     return greeter->priv->authentication_session;
 }
 
+gboolean
+greeter_get_resettable (Greeter *greeter)
+{
+    g_return_val_if_fail (greeter != NULL, FALSE);
+    return greeter->priv->resettable;
+}
+
 gboolean
 greeter_get_start_session (Greeter *greeter)
 {
@@ -903,6 +959,7 @@ greeter_start (Session *session)
     int to_greeter_output, from_greeter_input;
     gboolean result = FALSE;
     gchar *value;
+    GError *error = NULL;
 
     /* Create a pipe to talk with the greeter */
     if (pipe (to_greeter_pipe) != 0 || pipe (from_greeter_pipe) != 0)
@@ -913,11 +970,17 @@ greeter_start (Session *session)
     to_greeter_output = to_greeter_pipe[0];
     greeter->priv->to_greeter_input = to_greeter_pipe[1];
     greeter->priv->to_greeter_channel = g_io_channel_unix_new (greeter->priv->to_greeter_input);
-    g_io_channel_set_encoding (greeter->priv->to_greeter_channel, NULL, NULL);
+    g_io_channel_set_encoding (greeter->priv->to_greeter_channel, NULL, &error);
+    if (error)
+        g_warning ("Failed to set encoding on to greeter channel to binary: %s\n", error->message);
+    g_clear_error (&error);
     greeter->priv->from_greeter_output = from_greeter_pipe[0];
     from_greeter_input = from_greeter_pipe[1];
     greeter->priv->from_greeter_channel = g_io_channel_unix_new (greeter->priv->from_greeter_output);
-    g_io_channel_set_encoding (greeter->priv->from_greeter_channel, NULL, NULL);
+    g_io_channel_set_encoding (greeter->priv->from_greeter_channel, NULL, &error);
+    if (error)
+        g_warning ("Failed to set encoding on from greeter channel to binary: %s\n", error->message);
+    g_clear_error (&error);
     g_io_channel_set_buffered (greeter->priv->from_greeter_channel, FALSE);
     greeter->priv->from_greeter_watch = g_io_add_watch (greeter->priv->from_greeter_channel, G_IO_IN | G_IO_HUP, read_cb, greeter);
 
@@ -1049,7 +1112,7 @@ greeter_class_init (GreeterClass *klass)
     object_class->set_property = greeter_set_property;
 
     signals[CONNECTED] =
-        g_signal_new ("connected",
+        g_signal_new (GREETER_SIGNAL_CONNECTED,
                       G_TYPE_FROM_CLASS (klass),
                       G_SIGNAL_RUN_LAST,
                       G_STRUCT_OFFSET (GreeterClass, connected),
@@ -1058,7 +1121,7 @@ greeter_class_init (GreeterClass *klass)
                       G_TYPE_NONE, 0);
 
     signals[CREATE_SESSION] =
-        g_signal_new ("create-session",
+        g_signal_new (GREETER_SIGNAL_CREATE_SESSION,
                       G_TYPE_FROM_CLASS (klass),
                       G_SIGNAL_RUN_LAST,
                       G_STRUCT_OFFSET (GreeterClass, create_session),
@@ -1068,7 +1131,7 @@ greeter_class_init (GreeterClass *klass)
                       SESSION_TYPE, 0);
 
     signals[START_SESSION] =
-        g_signal_new ("start-session",
+        g_signal_new (GREETER_SIGNAL_START_SESSION,
                       G_TYPE_FROM_CLASS (klass),
                       G_SIGNAL_RUN_LAST,
                       G_STRUCT_OFFSET (GreeterClass, start_session),
@@ -1079,8 +1142,8 @@ greeter_class_init (GreeterClass *klass)
 
     g_object_class_install_property (object_class,
                                      PROP_ACTIVE_USERNAME,
-                                     g_param_spec_string ("active-username",
-                                                          "active-username",
+                                     g_param_spec_string (GREETER_PROPERTY_ACTIVE_USERNAME,
+                                                          GREETER_PROPERTY_ACTIVE_USERNAME,
                                                           "Active username",
                                                           NULL,
                                                           G_PARAM_READABLE));
