@@ -58,9 +58,14 @@ static gint exit_code = EXIT_SUCCESS;
 typedef struct
 {
     gchar *path;
-    gchar *parent_path;
     guint bus_id;
-} BusEntry;
+} SeatBusEntry;
+typedef struct
+{
+    gchar *path;
+    gchar *seat_path;
+    guint bus_id;
+} SessionBusEntry;
 
 #define LIGHTDM_BUS_NAME "org.freedesktop.DisplayManager"
 
@@ -139,32 +144,54 @@ log_init (void)
     g_free (path);
 }
 
-static void
-set_seat_properties (Seat *seat, const gchar *config_section)
+static GList*
+get_config_sections (const gchar *seat_name)
 {
+    gchar **groups, **i;
+    GList *config_sections = NULL;
+
+    config_sections = g_list_append (config_sections, g_strdup ("SeatDefaults"));
+
+    if (!seat_name)
+        return config_sections;
+
+    groups = config_get_groups (config_get_instance ());
+    for (i = groups; *i; i++)
+    {
+        if (g_str_has_prefix (*i, "Seat:"))
+        {
+            const gchar *seat_name_glob = *i + strlen ("Seat:");
+            if (g_pattern_match_simple (seat_name_glob, seat_name))
+                config_sections = g_list_append (config_sections, g_strdup (*i));
+        }
+    }
+    g_strfreev (groups);
+
+    return config_sections;
+}
+
+static void
+set_seat_properties (Seat *seat, const gchar *seat_name)
+{
+    GList *sections, *link;
     gchar **keys;
     gint i;
 
-    keys = config_get_keys (config_get_instance (), "SeatDefaults");
-    for (i = 0; keys[i]; i++)
+    sections = get_config_sections (seat_name);
+    for (link = sections; link; link = link->next)
     {
-        gchar *value = config_get_string (config_get_instance (), "SeatDefaults", keys[i]);
-        seat_set_property (seat, keys[i], value);
-        g_free (value);
-    }
-    g_strfreev (keys);
-
-    if (config_section)
-    {
-        keys = config_get_keys (config_get_instance (), config_section);
-        for (i = 0; keys[i]; i++)
+        const gchar *section = link->data;
+        g_debug ("Loading properties from config section %s", section);
+        keys = config_get_keys (config_get_instance (), section);
+        for (i = 0; keys && keys[i]; i++)
         {
-            gchar *value = config_get_string (config_get_instance (), config_section, keys[i]);
+            gchar *value = config_get_string (config_get_instance (), section, keys[i]);
             seat_set_property (seat, keys[i], value);
             g_free (value);
         }
         g_strfreev (keys);
     }
+    g_list_free_full (sections, g_free);
 }
 
 static void
@@ -201,7 +228,7 @@ display_manager_seat_removed_cb (DisplayManager *display_manager, Seat *seat)
 
         if (!next_seat)
         {
-            next_seat = seat_new (*iter);
+            next_seat = seat_new (*iter, seat_get_name (seat));
             g_string_assign (next_types, *iter);
         }
         else
@@ -215,14 +242,7 @@ display_manager_seat_removed_cb (DisplayManager *display_manager, Seat *seat)
 
     if (next_seat)
     {
-        const gchar *seat_name;
-        gchar *config_section = NULL;
-
-        seat_name = seat_get_string_property (seat, "seat-name");
-        if (seat_name)
-            config_section = g_strdup_printf ("Seat:%s", seat_name);
-        set_seat_properties (next_seat, config_section);
-        g_free (config_section);
+        set_seat_properties (next_seat, seat_get_name (seat));
 
         // We set this manually on default seat.  Let's port it over if needed.
         if (seat_get_boolean_property (seat, "exit-on-failure"))
@@ -243,6 +263,44 @@ display_manager_seat_removed_cb (DisplayManager *display_manager, Seat *seat)
     g_string_free (next_types, TRUE);
 }
 
+static GVariant *
+get_seat_list (void)
+{
+    GVariantBuilder builder;
+    GHashTableIter iter;
+    gpointer value;
+
+    g_variant_builder_init (&builder, G_VARIANT_TYPE ("ao"));
+    g_hash_table_iter_init (&iter, seat_bus_entries);
+    while (g_hash_table_iter_next (&iter, NULL, &value))
+    {
+        SeatBusEntry *entry = value;
+        g_variant_builder_add_value (&builder, g_variant_new_object_path (entry->path));
+    }
+
+    return g_variant_builder_end (&builder);
+}
+
+static GVariant *
+get_session_list (const gchar *seat_path)
+{
+    GVariantBuilder builder;
+    GHashTableIter iter;
+    gpointer value;
+
+    g_variant_builder_init (&builder, G_VARIANT_TYPE ("ao"));
+
+    g_hash_table_iter_init (&iter, session_bus_entries);
+    while (g_hash_table_iter_next (&iter, NULL, &value))
+    {
+        SessionBusEntry *entry = value;
+        if (seat_path == NULL || strcmp (entry->seat_path, seat_path) == 0)
+            g_variant_builder_add_value (&builder, g_variant_new_object_path (entry->path));
+    }
+
+    return g_variant_builder_end (&builder);
+}
+
 static GVariant *
 handle_display_manager_get_property (GDBusConnection       *connection,
                                      const gchar           *sender,
@@ -252,42 +310,12 @@ handle_display_manager_get_property (GDBusConnection       *connection,
                                      GError               **error,
                                      gpointer               user_data)
 {
-    GVariant *result = NULL;
-
     if (g_strcmp0 (property_name, "Seats") == 0)
-    {
-        GVariantBuilder *builder;
-        GHashTableIter iter;
-        gpointer value;
-
-        builder = g_variant_builder_new (G_VARIANT_TYPE ("ao"));
-        g_hash_table_iter_init (&iter, seat_bus_entries);
-        while (g_hash_table_iter_next (&iter, NULL, &value))
-        {
-            BusEntry *entry = value;
-            g_variant_builder_add_value (builder, g_variant_new_object_path (entry->path));
-        }
-        result = g_variant_builder_end (builder);
-        g_variant_builder_unref (builder);
-    }
+        return get_seat_list ();
     else if (g_strcmp0 (property_name, "Sessions") == 0)
-    {
-        GVariantBuilder *builder;
-        GHashTableIter iter;
-        gpointer value;
+        return get_session_list (NULL);
 
-        builder = g_variant_builder_new (G_VARIANT_TYPE ("ao"));
-        g_hash_table_iter_init (&iter, session_bus_entries);
-        while (g_hash_table_iter_next (&iter, NULL, &value))
-        {
-            BusEntry *entry = value;
-            g_variant_builder_add_value (builder, g_variant_new_object_path (entry->path));
-        }
-        result = g_variant_builder_end (builder);
-        g_variant_builder_unref (builder);
-    }
-
-    return result;
+    return NULL;
 }
 
 static void
@@ -301,59 +329,23 @@ handle_display_manager_call (GDBusConnection       *connection,
                              gpointer               user_data)
 {
     if (g_strcmp0 (method_name, "AddSeat") == 0)
-    {
-        gchar *type;
-        GVariantIter *property_iter;
-        gchar *name, *value;
-        Seat *seat;
-
-        if (!g_variant_is_of_type (parameters, G_VARIANT_TYPE ("(sa(ss))")))
-            return;
-
-        g_variant_get (parameters, "(&sa(ss))", &type, &property_iter);
-
-        g_debug ("Adding seat of type %s", type);
-
-        seat = seat_new (type);
-        if (seat)
-        {
-            set_seat_properties (seat, NULL);
-            while (g_variant_iter_loop (property_iter, "(&s&s)", &name, &value))
-                seat_set_property (seat, name, value);
-        }
-        g_variant_iter_free (property_iter);
-
-        if (!seat)
-        {
-            // FIXME: Need to make proper error
-            g_dbus_method_invocation_return_error (invocation, G_DBUS_ERROR, G_DBUS_ERROR_FAILED, "Unable to create seat of type %s", type);
-            return;
-        }
-
-        if (display_manager_add_seat (display_manager, seat))
-        {
-            BusEntry *entry;
-
-            entry = g_hash_table_lookup (seat_bus_entries, seat);
-            g_dbus_method_invocation_return_value (invocation, g_variant_new ("(o)", entry->path));
-        }
-        else// FIXME: Need to make proper error
-            g_dbus_method_invocation_return_error (invocation, G_DBUS_ERROR, G_DBUS_ERROR_FAILED, "Failed to start seat");
-        g_object_unref (seat);
-    }
+        g_dbus_method_invocation_return_error (invocation, G_DBUS_ERROR, G_DBUS_ERROR_INVALID_ARGS, "AddSeat is deprecated");
     else if (g_strcmp0 (method_name, "AddLocalXSeat") == 0)
     {
         gint display_number;
         Seat *seat;
 
         if (!g_variant_is_of_type (parameters, G_VARIANT_TYPE ("(i)")))
+        {
+            g_dbus_method_invocation_return_error (invocation, G_DBUS_ERROR, G_DBUS_ERROR_INVALID_ARGS, "Invalid arguments");
             return;
+        }
 
         g_variant_get (parameters, "(i)", &display_number);
 
         g_debug ("Adding local X seat :%d", display_number);
 
-        seat = seat_new ("xremote");
+        seat = seat_new ("xremote", "xremote0"); // FIXME: What to use for a name?
         if (seat)
         {
             gchar *display_number_string;
@@ -373,7 +365,7 @@ handle_display_manager_call (GDBusConnection       *connection,
 
         if (display_manager_add_seat (display_manager, seat))
         {
-            BusEntry *entry;
+            SeatBusEntry *entry;
 
             entry = g_hash_table_lookup (seat_bus_entries, seat);
             g_dbus_method_invocation_return_value (invocation, g_variant_new ("(o)", entry->path));
@@ -382,6 +374,8 @@ handle_display_manager_call (GDBusConnection       *connection,
             g_dbus_method_invocation_return_error (invocation, G_DBUS_ERROR, G_DBUS_ERROR_FAILED, "Failed to start seat");
         g_object_unref (seat);
     }
+    else
+        g_dbus_method_invocation_return_error (invocation, G_DBUS_ERROR, G_DBUS_ERROR_UNKNOWN_METHOD, "Unknown method");
 }
 
 static GVariant *
@@ -394,31 +388,20 @@ handle_seat_get_property (GDBusConnection       *connection,
                           gpointer               user_data)
 {
     Seat *seat = user_data;
-    GVariant *result = NULL;
 
     if (g_strcmp0 (property_name, "CanSwitch") == 0)
-        result = g_variant_new_boolean (seat_get_can_switch (seat));
+        return g_variant_new_boolean (seat_get_can_switch (seat));
     if (g_strcmp0 (property_name, "HasGuestAccount") == 0)
-        result = g_variant_new_boolean (seat_get_allow_guest (seat));
+        return g_variant_new_boolean (seat_get_allow_guest (seat));
     else if (g_strcmp0 (property_name, "Sessions") == 0)
     {
-        GVariantBuilder *builder;
-        GList *link;
+        SeatBusEntry *entry;
 
-        builder = g_variant_builder_new (G_VARIANT_TYPE ("ao"));
-        for (link = seat_get_sessions (seat); link; link = link->next)
-        {
-            Session *session = link->data;
-            BusEntry *entry;
-            entry = g_hash_table_lookup (session_bus_entries, session);
-            if (entry)
-                g_variant_builder_add_value (builder, g_variant_new_object_path (entry->path));
-        }
-        result = g_variant_builder_end (builder);
-        g_variant_builder_unref (builder);
+        entry = g_hash_table_lookup (seat_bus_entries, seat);
+        return get_session_list (entry->path);
     }
 
-    return result;
+    return NULL;
 }
 
 static void
@@ -522,11 +505,11 @@ handle_session_get_property (GDBusConnection       *connection,
                              gpointer               user_data)
 {
     Session *session = user_data;
-    BusEntry *entry;
+    SessionBusEntry *entry;
 
     entry = g_hash_table_lookup (session_bus_entries, session);
     if (g_strcmp0 (property_name, "Seat") == 0)
-        return g_variant_new_object_path (entry ? entry->parent_path : "");
+        return g_variant_new_object_path (entry ? entry->seat_path : "");
     else if (g_strcmp0 (property_name, "UserName") == 0)
         return g_variant_new_string (session_get_username (session));
 
@@ -549,32 +532,93 @@ handle_session_call (GDBusConnection       *connection,
     {
         Seat *seat;
 
+        if (!g_variant_is_of_type (parameters, G_VARIANT_TYPE ("()")))
+            g_dbus_method_invocation_return_error (invocation, G_DBUS_ERROR, G_DBUS_ERROR_INVALID_ARGS, "Invalid arguments");
+
         seat = get_seat_for_session (session);
         /* FIXME: Should only allow locks if have a session on this seat */
         seat_lock (seat, session_get_username (session));
         g_dbus_method_invocation_return_value (invocation, NULL);
     }
+    else
+        g_dbus_method_invocation_return_error (invocation, G_DBUS_ERROR, G_DBUS_ERROR_UNKNOWN_METHOD, "Unknown method");
 }
 
-static BusEntry *
-bus_entry_new (const gchar *path, const gchar *parent_path)
+static SeatBusEntry *
+seat_bus_entry_new (const gchar *path)
 {
-    BusEntry *entry;
+    SeatBusEntry *entry;
 
-    entry = g_malloc0 (sizeof (BusEntry));
+    entry = g_malloc0 (sizeof (SeatBusEntry));
     entry->path = g_strdup (path);
-    entry->parent_path = g_strdup (parent_path);
+
+    return entry;
+}
+
+static SessionBusEntry *
+session_bus_entry_new (const gchar *path, const gchar *seat_path)
+{
+    SessionBusEntry *entry;
+
+    entry = g_malloc0 (sizeof (SessionBusEntry));
+    entry->path = g_strdup (path);
+    entry->seat_path = g_strdup (seat_path);
 
     return entry;
 }
 
 static void
-bus_entry_free (gpointer data)
+emit_object_value_changed (GDBusConnection *bus, const gchar *path, const gchar *interface_name, const gchar *property_name, GVariant *property_value)
 {
-    BusEntry *entry = data;
+    GVariantBuilder builder;
+    GError *error = NULL;
+
+    g_variant_builder_init (&builder, G_VARIANT_TYPE_ARRAY);
+    g_variant_builder_add (&builder, "{sv}", property_name, property_value);
+
+    if (!g_dbus_connection_emit_signal (bus,
+                                        NULL,
+                                        path,
+                                        "org.freedesktop.DBus.Properties",
+                                        "PropertiesChanged",
+                                        g_variant_new ("(sa{sv}as)", interface_name, &builder, NULL),
+                                        &error))
+        g_warning ("Failed to emit PropertiesChanged signal: %s", error->message);
+    g_clear_error (&error);
+}
+
+static void
+emit_object_signal (GDBusConnection *bus, const gchar *path, const gchar *signal_name, const gchar *object_path)
+{
+    GError *error = NULL;
+
+    if (!g_dbus_connection_emit_signal (bus,
+                                        NULL,
+                                        path,
+                                        "org.freedesktop.DisplayManager",
+                                        signal_name,
+                                        g_variant_new ("(o)", object_path),
+                                        &error))
+        g_warning ("Failed to emit %s signal on %s: %s", signal_name, path, error->message);
+    g_clear_error (&error);
+}
+
+static void
+seat_bus_entry_free (gpointer data)
+{
+    SeatBusEntry *entry = data;
 
     g_free (entry->path);
-    g_free (entry->parent_path);
+    g_free (entry);
+}
+
+static void
+session_bus_entry_free (gpointer data)
+{
+    SessionBusEntry *entry = data;
+
+    g_free (entry->path);
+    g_free (entry->seat_path);
     g_free (entry);
 }
 
@@ -586,7 +630,8 @@ running_user_session_cb (Seat *seat, Session *session)
         handle_session_call,
         handle_session_get_property
     };
-    BusEntry *seat_entry, *entry;
+    SeatBusEntry *seat_entry;
+    SessionBusEntry *session_entry;
     gchar *path;
     GError *error = NULL;
 
@@ -598,58 +643,53 @@ running_user_session_cb (Seat *seat, Session *session)
     session_set_env (session, "XDG_SESSION_PATH", path);
     g_object_set_data_full (G_OBJECT (session), "XDG_SESSION_PATH", path, g_free);
 
-    seat_entry = g_hash_table_lookup (seat_bus_entries, seat);
-    entry = bus_entry_new (g_object_get_data (G_OBJECT (session), "XDG_SESSION_PATH"), seat_entry ? seat_entry->path : NULL);
-    g_hash_table_insert (session_bus_entries, g_object_ref (session), entry);
+    session_entry = session_bus_entry_new (g_object_get_data (G_OBJECT (session), "XDG_SESSION_PATH"), seat_entry ? seat_entry->path : NULL);
+    g_hash_table_insert (session_bus_entries, g_object_ref (session), session_entry);
 
-    g_debug ("Registering session with bus path %s", entry->path);
+    g_debug ("Registering session with bus path %s", session_entry->path);
 
-    entry->bus_id = g_dbus_connection_register_object (bus,
-                                                       entry->path,
-                                                       session_info->interfaces[0],
-                                                       &session_vtable,
-                                                       g_object_ref (session), g_object_unref,
-                                                       &error);
-    if (entry->bus_id == 0)
+    session_entry->bus_id = g_dbus_connection_register_object (bus,
+                                                               session_entry->path,
+                                                               session_info->interfaces[0],
+                                                               &session_vtable,
+                                                               g_object_ref (session), g_object_unref,
+                                                               &error);
+    if (session_entry->bus_id == 0)
         g_warning ("Failed to register user session: %s", error->message);
     g_clear_error (&error);
 
-    if (!g_dbus_connection_emit_signal (bus,
-                                        NULL,
-                                        "/org/freedesktop/DisplayManager",
-                                        "org.freedesktop.DisplayManager",
-                                        "SessionAdded",
-                                        g_variant_new ("(o)", entry->path),
-                                        &error))
-        g_warning ("Failed to emit SessionAdded signal: %s", error->message);
-    g_clear_error (&error);
+    emit_object_value_changed (bus, "/org/freedesktop/DisplayManager", "org.freedesktop.DisplayManager", "Sessions", get_session_list (NULL));
+    emit_object_signal (bus, "/org/freedesktop/DisplayManager", "SessionAdded", session_entry->path);
+
+    emit_object_value_changed (bus, seat_entry->path, "org.freedesktop.DisplayManager.Seat", "Sessions", get_session_list (session_entry->seat_path));
+    emit_object_signal (bus, seat_entry->path, "SessionAdded", session_entry->path);
 }
 
 static void
 session_removed_cb (Seat *seat, Session *session)
 {
-    BusEntry *entry;
+    SessionBusEntry *entry;
+    gchar *seat_path = NULL;
 
     g_signal_handlers_disconnect_matched (session, G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, seat);
 
     entry = g_hash_table_lookup (session_bus_entries, session);
     if (entry)
     {
-        GError *error = NULL;
-
         g_dbus_connection_unregister_object (bus, entry->bus_id);
-        if (!g_dbus_connection_emit_signal (bus,
-                                            NULL,
-                                            "/org/freedesktop/DisplayManager",
-                                            "org.freedesktop.DisplayManager",
-                                            "SessionRemoved",
-                                            g_variant_new ("(o)", entry->path),
-                                            &error))
-            g_warning ("Failed to emit SessionRemoved signal: %s", error->message);
-        g_clear_error (&error);
+        emit_object_signal (bus, "/org/freedesktop/DisplayManager", "SessionRemoved", entry->path);
+        emit_object_signal (bus, entry->seat_path, "SessionRemoved", entry->path);
+        seat_path = g_strdup (entry->seat_path);
     }
 
     g_hash_table_remove (session_bus_entries, session);
+
+    if (seat_path)
+    {
+        emit_object_value_changed (bus, "/org/freedesktop/DisplayManager", "org.freedesktop.DisplayManager", "Sessions", get_session_list (NULL));
+        emit_object_value_changed (bus, seat_path, "org.freedesktop.DisplayManager.Seat", "Sessions", get_session_list (seat_path));
+        g_free (seat_path);
+    }
 }
 
 static void
@@ -661,13 +701,13 @@ seat_added_cb (DisplayManager *display_manager, Seat *seat)
         handle_seat_get_property
     };
     gchar *path;
-    BusEntry *entry;
+    SeatBusEntry *entry;
     GError *error = NULL;
 
     path = g_strdup_printf ("/org/freedesktop/DisplayManager/Seat%d", seat_index);
     seat_index++;
 
-    entry = bus_entry_new (path, NULL);
+    entry = seat_bus_entry_new (path);
     g_free (path);
     g_hash_table_insert (seat_bus_entries, g_object_ref (seat), entry);
 
@@ -683,43 +723,28 @@ seat_added_cb (DisplayManager *display_manager, Seat *seat)
         g_warning ("Failed to register seat: %s", error->message);
     g_clear_error (&error);
 
-    if (!g_dbus_connection_emit_signal (bus,
-                                        NULL,
-                                        "/org/freedesktop/DisplayManager",
-                                        "org.freedesktop.DisplayManager",
-                                        "SeatAdded",
-                                        g_variant_new ("(o)", entry->path),
-                                        &error))
-        g_warning ("Failed to emit SeatAdded signal: %s", error->message);
-    g_clear_error (&error);
+    emit_object_value_changed (bus, "/org/freedesktop/DisplayManager", "org.freedesktop.DisplayManager", "Seats", get_seat_list ());
+    emit_object_signal (bus, "/org/freedesktop/DisplayManager", "SeatAdded", entry->path);
 
-    g_signal_connect (seat, "running-user-session", G_CALLBACK (running_user_session_cb), NULL);
-    g_signal_connect (seat, "session-removed", G_CALLBACK (session_removed_cb), NULL);
+    g_signal_connect (seat, SEAT_SIGNAL_RUNNING_USER_SESSION, G_CALLBACK (running_user_session_cb), NULL);
+    g_signal_connect (seat, SEAT_SIGNAL_SESSION_REMOVED, G_CALLBACK (session_removed_cb), NULL);
 }
 
 static void
 seat_removed_cb (DisplayManager *display_manager, Seat *seat)
 {
-    BusEntry *entry;
+    SeatBusEntry *entry;
 
     entry = g_hash_table_lookup (seat_bus_entries, seat);
     if (entry)
     {
-        GError *error = NULL;
-
         g_dbus_connection_unregister_object (bus, entry->bus_id);
-        if (!g_dbus_connection_emit_signal (bus,
-                                            NULL,
-                                            "/org/freedesktop/DisplayManager",
-                                            "org.freedesktop.DisplayManager",
-                                            "SeatRemoved",
-                                            g_variant_new ("(o)", entry->path),
-                                            &error))
-            g_warning ("Failed to emit SeatRemoved signal: %s", error->message);
-        g_clear_error (&error);
+        emit_object_signal (bus, "/org/freedesktop/DisplayManager", "SeatRemoved", entry->path);
     }
 
     g_hash_table_remove (seat_bus_entries, seat);
+  
+    emit_object_value_changed (bus, "/org/freedesktop/DisplayManager", "org.freedesktop.DisplayManager", "Seats", get_seat_list ());
 }
 
 static gboolean
@@ -800,6 +825,12 @@ bus_acquired_cb (GDBusConnection *connection,
         "      <arg name='session-name' direction='in' type='s'/>"
         "    </method>"
         "    <method name='Lock'/>"
+        "    <signal name='SessionAdded'>"
+        "      <arg name='session' type='o'/>"
+        "    </signal>"
+        "    <signal name='SessionRemoved'>"
+        "      <arg name='session' type='o'/>"
+        "    </signal>"
         "  </interface>"
         "</node>";
     const gchar *session_interface =
@@ -836,11 +867,11 @@ bus_acquired_cb (GDBusConnection *connection,
     g_clear_error (&error);
     g_dbus_node_info_unref (display_manager_info);
 
-    seat_bus_entries = g_hash_table_new_full (g_direct_hash, g_direct_equal, g_object_unref, bus_entry_free);
-    session_bus_entries = g_hash_table_new_full (g_direct_hash, g_direct_equal, g_object_unref, bus_entry_free);
+    seat_bus_entries = g_hash_table_new_full (g_direct_hash, g_direct_equal, g_object_unref, seat_bus_entry_free);
+    session_bus_entries = g_hash_table_new_full (g_direct_hash, g_direct_equal, g_object_unref, session_bus_entry_free);
 
-    g_signal_connect (display_manager, "seat-added", G_CALLBACK (seat_added_cb), NULL);
-    g_signal_connect (display_manager, "seat-removed", G_CALLBACK (seat_removed_cb), NULL);
+    g_signal_connect (display_manager, DISPLAY_MANAGER_SIGNAL_SEAT_ADDED, G_CALLBACK (seat_added_cb), NULL);
+    g_signal_connect (display_manager, DISPLAY_MANAGER_SIGNAL_SEAT_REMOVED, G_CALLBACK (seat_removed_cb), NULL);
     for (link = display_manager_get_seats (display_manager); link; link = link->next)
         seat_added_cb (display_manager, (Seat *) link->data);
 
@@ -862,7 +893,7 @@ bus_acquired_cb (GDBusConnection *connection,
         listen_address = config_get_string (config_get_instance (), "XDMCPServer", "listen-address");
         xdmcp_server_set_listen_address (xdmcp_server, listen_address);
         g_free (listen_address);
-        g_signal_connect (xdmcp_server, "new-session", G_CALLBACK (xdmcp_session_cb), NULL);
+        g_signal_connect (xdmcp_server, XDMCP_SERVER_SIGNAL_NEW_SESSION, G_CALLBACK (xdmcp_session_cb), NULL);
 
         key_name = config_get_string (config_get_instance (), "XDMCPServer", "key");
         if (key_name)
@@ -929,7 +960,7 @@ bus_acquired_cb (GDBusConnection *connection,
             listen_address = config_get_string (config_get_instance (), "VNCServer", "listen-address");
             vnc_server_set_listen_address (vnc_server, listen_address);
             g_free (listen_address);
-            g_signal_connect (vnc_server, "new-connection", G_CALLBACK (vnc_connection_cb), NULL);
+            g_signal_connect (vnc_server, VNC_SERVER_SIGNAL_NEW_CONNECTION, G_CALLBACK (vnc_connection_cb), NULL);
 
             g_debug ("Starting VNC server on TCP/IP port %d", vnc_server_get_port (vnc_server));
             vnc_server_start (vnc_server);
@@ -958,43 +989,31 @@ static gboolean
 add_login1_seat (Login1Seat *login1_seat)
 {
     const gchar *seat_name = login1_seat_get_id (login1_seat);
-    gchar **groups, **i;
-    gchar *config_section = NULL;
     gchar **types = NULL, **type;
+    GList *config_sections = NULL, *link;
     Seat *seat = NULL;
     gboolean is_seat0, started = FALSE;
 
     g_debug ("New seat added from logind: %s", seat_name);
     is_seat0 = strcmp (seat_name, "seat0") == 0;
 
-    groups = config_get_groups (config_get_instance ());
-    for (i = groups; !config_section && *i; i++)
+    config_sections = get_config_sections (seat_name);
+    for (link = g_list_last (config_sections); link; link = link->prev)
     {
-        if (g_str_has_prefix (*i, "Seat:") &&
-            g_str_has_suffix (*i, seat_name))
-        {
-            config_section = g_strdup (*i);
-            break;
-        }
-    }
-    g_strfreev (groups);
-
-    if (config_section)
-    {
-        g_debug ("Loading properties from config section %s", config_section);
+        gchar *config_section = link->data;
         types = config_get_string_list (config_get_instance (), config_section, "type");
+        if (types)
+            break;
     }
+    g_list_free_full (config_sections, g_free);
 
-    if (!types)
-        types = config_get_string_list (config_get_instance (), "SeatDefaults", "type");
     for (type = types; !seat && type && *type; type++)
-        seat = seat_new (*type);
+        seat = seat_new (*type, seat_name);
     g_strfreev (types);
 
     if (seat)
     {
-        seat_set_property (seat, "xdg-seat", seat_name);
-        set_seat_properties (seat, NULL);
+        set_seat_properties (seat, seat_name);
 
         if (!login1_seat_get_can_multi_session (login1_seat))
         {
@@ -1003,9 +1022,6 @@ add_login1_seat (Login1Seat *login1_seat)
             seat_set_property (seat, "allow-user-switching", "false"); */
         }
 
-        if (config_section)
-            set_seat_properties (seat, config_section);
-
         if (is_seat0)
             seat_set_property (seat, "exit-on-failure", "true");
     }
@@ -1019,9 +1035,8 @@ add_login1_seat (Login1Seat *login1_seat)
             g_debug ("Failed to start seat: %s", seat_name);
     }
 
-    g_free (config_section);
     g_object_unref (seat);
-  
+
     return started;
 }
 
@@ -1055,7 +1070,7 @@ update_login1_seat (Login1Seat *login1_seat)
         if (seat)
         {
             if (seat_get_is_stopping (seat))
-                g_signal_connect (seat, "stopped", G_CALLBACK (seat_stopped_cb), login1_seat);
+                g_signal_connect (seat, SEAT_SIGNAL_STOPPED, G_CALLBACK (seat_stopped_cb), login1_seat);
             return TRUE;
         }
 
@@ -1075,6 +1090,48 @@ login1_can_graphical_changed_cb (Login1Seat *login1_seat)
     update_login1_seat (login1_seat);
 }
 
+static void
+login1_active_session_changed_cb (Login1Seat *login1_seat, const gchar *login1_session_id)
+{
+    g_debug ("Seat %s changes active session to %s", login1_seat_get_id (login1_seat), login1_session_id);
+
+    Seat *seat;
+    seat = display_manager_get_seat (display_manager, login1_seat_get_id (login1_seat));
+
+    if (seat)
+    {
+        Session *active_session;
+        active_session = seat_get_expected_active_session (seat);
+
+        if (g_strcmp0 (login1_session_id, session_get_login1_session_id (active_session)) == 0)
+        {
+            // Session is already active
+            g_debug ("Session %s is already active", login1_session_id);
+            return;
+        }
+
+        active_session = seat_find_session_by_login1_id (seat, login1_session_id);
+        if (active_session != NULL)
+        {
+            g_debug ("Activating session %s", login1_session_id);
+            seat_set_externally_activated_session (seat, active_session);
+            return;
+
+        }
+    }
+}
+
+static gboolean
+login1_add_seat (Login1Seat *login1_seat)
+{
+    if (config_get_boolean (config_get_instance (), "LightDM", "logind-check-graphical"))
+        g_signal_connect (login1_seat, "can-graphical-changed", G_CALLBACK (login1_can_graphical_changed_cb), NULL);
+
+    g_signal_connect (login1_seat, LOGIN1_SIGNAL_ACTIVE_SESION_CHANGED, G_CALLBACK (login1_active_session_changed_cb), NULL);
+
+    return update_login1_seat (login1_seat);
+}
+
 static void
 login1_service_seat_added_cb (Login1Service *service, Login1Seat *login1_seat)
 {
@@ -1083,9 +1140,7 @@ login1_service_seat_added_cb (Login1Service *service, Login1Seat *login1_seat)
     else
         g_debug ("Seat %s added from logind without graphical output", login1_seat_get_id (login1_seat));
 
-    if (config_get_boolean (config_get_instance (), "LightDM", "logind-check-graphical"))
-        g_signal_connect (login1_seat, "can-graphical-changed", G_CALLBACK (login1_can_graphical_changed_cb), NULL);
-    update_login1_seat (login1_seat);
+    login1_add_seat (login1_seat);
 }
 
 static void
@@ -1093,6 +1148,7 @@ login1_service_seat_removed_cb (Login1Service *service, Login1Seat *login1_seat)
 {
     g_debug ("Seat %s removed from logind", login1_seat_get_id (login1_seat));
     g_signal_handlers_disconnect_matched (login1_seat, G_SIGNAL_MATCH_FUNC, 0, 0, NULL, login1_can_graphical_changed_cb, NULL);
+    g_signal_handlers_disconnect_matched (login1_seat, G_SIGNAL_MATCH_FUNC, 0, 0, NULL, login1_active_session_changed_cb, NULL);
     remove_login1_seat (login1_seat);
 }
 
@@ -1113,7 +1169,6 @@ main (int argc, char **argv)
     gchar *default_cache_dir = g_strdup (CACHE_DIR);
     gboolean show_config = FALSE, show_version = FALSE;
     GList *link, *messages = NULL;
-    gboolean connected;
     GOptionEntry options[] =
     {
         { "config", 'c', 0, G_OPTION_ARG_STRING, &config_path,
@@ -1158,7 +1213,7 @@ main (int argc, char **argv)
 
     messages = g_list_append (messages, g_strdup_printf ("Starting Light Display Manager %s, UID=%i PID=%i", VERSION, getuid (), getpid ()));
 
-    g_signal_connect (process_get_current (), "got-signal", G_CALLBACK (signal_cb), NULL);
+    g_signal_connect (process_get_current (), PROCESS_SIGNAL_GOT_SIGNAL, G_CALLBACK (signal_cb), NULL);
 
     option_context = g_option_context_new (/* Arguments and description for --help test */
                                            _("- Display Manager"));
@@ -1219,7 +1274,7 @@ main (int argc, char **argv)
             g_printerr ("%s  [%s]\n", empty_source, groups[i]);
 
             keys = config_get_keys (config_get_instance (), groups[i]);
-            for (j = 0; keys[j]; j++)
+            for (j = 0; keys && keys[j]; j++)
             {
                 const gchar *source, *id;
                 gchar *value;
@@ -1426,116 +1481,33 @@ main (int argc, char **argv)
         g_debug ("Using Xephyr for X servers");
 
     display_manager = display_manager_new ();
-    g_signal_connect (display_manager, "stopped", G_CALLBACK (display_manager_stopped_cb), NULL);
-    g_signal_connect (display_manager, "seat-removed", G_CALLBACK (display_manager_seat_removed_cb), NULL);
+    g_signal_connect (display_manager, DISPLAY_MANAGER_SIGNAL_STOPPED, G_CALLBACK (display_manager_stopped_cb), NULL);
+    g_signal_connect (display_manager, DISPLAY_MANAGER_SIGNAL_SEAT_REMOVED, G_CALLBACK (display_manager_seat_removed_cb), NULL);
 
     shared_data_manager_start (shared_data_manager_get_instance ());
 
     /* Connect to logind */
-    connected = login1_service_connect (login1_service_get_instance ());
-
-    /* Load dynamic seats from logind */
-    if (config_get_boolean (config_get_instance (), "LightDM", "logind-load-seats"))
+    if (login1_service_connect (login1_service_get_instance ()))
     {
-        if (connected)
+        /* Load dynamic seats from logind */
+        g_debug ("Monitoring logind for seats");
+
+        if (config_get_boolean (config_get_instance (), "LightDM", "start-default-seat"))
         {
-            /* Load dynamic seats from logind */
-            g_debug ("Monitoring logind for seats");
+            g_signal_connect (login1_service_get_instance (), LOGIN1_SERVICE_SIGNAL_SEAT_ADDED, G_CALLBACK (login1_service_seat_added_cb), NULL);
+            g_signal_connect (login1_service_get_instance (), LOGIN1_SERVICE_SIGNAL_SEAT_REMOVED, G_CALLBACK (login1_service_seat_removed_cb), NULL);
 
-            if (config_get_boolean (config_get_instance (), "LightDM", "start-default-seat"))
+            for (link = login1_service_get_seats (login1_service_get_instance ()); link; link = link->next)
             {
-                g_signal_connect (login1_service_get_instance (), "seat-added", G_CALLBACK (login1_service_seat_added_cb), NULL);
-                g_signal_connect (login1_service_get_instance (), "seat-removed", G_CALLBACK (login1_service_seat_removed_cb), NULL);
-
-                for (link = login1_service_get_seats (login1_service_get_instance ()); link; link = link->next)
-                {
-                    Login1Seat *login1_seat = link->data;
-                    if (config_get_boolean (config_get_instance (), "LightDM", "logind-check-graphical"))
-                        g_signal_connect (login1_seat, "can-graphical-changed", G_CALLBACK (login1_can_graphical_changed_cb), NULL);
-                    if (!update_login1_seat (login1_seat))
-                        return EXIT_FAILURE;
-                }
-            }
-        }
-        else if (config_get_boolean (config_get_instance (), "LightDM", "start-default-seat"))
-        {
-            gchar **types;
-            gchar **type;
-            Seat *seat = NULL;
-
-            g_debug ("Adding default seat");
-
-            types = config_get_string_list (config_get_instance (), "SeatDefaults", "type");
-            for (type = types; type && *type; type++)
-            {
-                seat = seat_new (*type);
-                if (seat)
-                    break;
-            }
-            g_strfreev (types);
-            if (seat)
-            {
-                set_seat_properties (seat, NULL);
-                seat_set_property (seat, "exit-on-failure", "true");
-                if (!display_manager_add_seat (display_manager, seat))
+                Login1Seat *login1_seat = link->data;
+                if (!login1_add_seat (login1_seat))
                     return EXIT_FAILURE;
-                g_object_unref (seat);
-            }
-            else
-            {
-                g_warning ("Failed to create default seat");
-                return EXIT_FAILURE;
             }
         }
     }
     else
     {
-        gchar **groups, **i;
-        gint n_seats = 0;
-
-        /* Load the static seat entries */
-        groups = config_get_groups (config_get_instance ());
-        for (i = groups; *i; i++)
-        {
-            gchar *config_section = *i;
-            gchar **types;
-            gchar **type;
-            Seat *seat = NULL;
-            const gchar *const seatpfx = "Seat:";
-
-            if (!g_str_has_prefix (config_section, seatpfx))
-                continue;
-
-            g_debug ("Loading seat %s", config_section);
-            types = config_get_string_list (config_get_instance (), config_section, "type");
-            if (!types)
-                types = config_get_string_list (config_get_instance (), "SeatDefaults", "type");
-            for (type = types; type && *type; type++)
-            {
-                seat = seat_new (*type);
-                if (seat)
-                    break;
-            }
-            g_strfreev (types);
-            if (seat)
-            {
-                const gsize seatpfxlen = strlen(seatpfx);
-                gchar *seatname = config_section + seatpfxlen;
-
-                seat_set_property (seat, "seat-name", seatname);
-
-                set_seat_properties (seat, config_section);
-                display_manager_add_seat (display_manager, seat);
-                g_object_unref (seat);
-                n_seats++;
-            }
-            else
-                g_warning ("Failed to create seat %s", config_section);
-        }
-        g_strfreev (groups);
-
-        /* If no seats start a default one */
-        if (n_seats == 0 && config_get_boolean (config_get_instance (), "LightDM", "start-default-seat"))
+        if (config_get_boolean (config_get_instance (), "LightDM", "start-default-seat"))
         {
             gchar **types;
             gchar **type;
@@ -1546,7 +1518,7 @@ main (int argc, char **argv)
             types = config_get_string_list (config_get_instance (), "SeatDefaults", "type");
             for (type = types; type && *type; type++)
             {
-                seat = seat_new (*type);
+                seat = seat_new (*type, "seat0");
                 if (seat)
                     break;
             }
