@@ -1,7 +1,7 @@
 /*
  * Copyright (C) 2010-2011 Robert Ancell.
  * Author: Robert Ancell <robert.ancell@canonical.com>
- * 
+ *
  * This program is free software: you can redistribute it and/or modify it under
  * the terms of the GNU General Public License as published by the Free Software
  * Foundation, either version 3 of the License, or (at your option) any later
@@ -30,6 +30,9 @@ static guint signals[LAST_SIGNAL] = { 0 };
 
 struct SeatPrivate
 {
+    /* XDG name for this seat */
+    gchar *name;
+
     /* Configuration for this seat */
     GHashTable *properties;
 
@@ -53,7 +56,7 @@ struct SeatPrivate
 
     /* The session to set active when it starts */
     Session *session_to_activate;
-  
+
     /* TRUE once we have started */
     gboolean started;
 
@@ -99,11 +102,11 @@ seat_register_module (const gchar *name, GType type)
 }
 
 Seat *
-seat_new (const gchar *module_name)
+seat_new (const gchar *module_name, const gchar *name)
 {
     Seat *seat;
     SeatModule *m = NULL;
-  
+
     g_return_val_if_fail (module_name != NULL, NULL);
 
     if (seat_modules)
@@ -112,6 +115,7 @@ seat_new (const gchar *module_name)
         return NULL;
 
     seat = g_object_new (m->type, NULL);
+    seat->priv->name = g_strdup (name);
 
     return seat;
 }
@@ -167,13 +171,7 @@ seat_get_integer_property (Seat *seat, const gchar *name)
 const gchar *
 seat_get_name (Seat *seat)
 {
-    const gchar *name;
-
-    name = seat_get_string_property (seat, "xdg-seat");
-    if (name)
-        return name;
-
-    return "seat0";
+    return seat->priv->name;
 }
 
 void
@@ -210,6 +208,13 @@ seat_get_sessions (Seat *seat)
     return seat->priv->sessions;
 }
 
+static gboolean
+set_greeter_idle (gpointer greeter)
+{
+    greeter_idle (GREETER (greeter));
+    return FALSE;
+}
+
 void
 seat_set_active_session (Seat *seat, Session *session)
 {
@@ -229,8 +234,24 @@ seat_set_active_session (Seat *seat, Session *session)
 
         if (IS_GREETER (s))
         {
-            l_debug (seat, "Stopping greeter");
-            session_stop (s);
+            Greeter *greeter = GREETER (s);
+            if (greeter_get_resettable (greeter))
+            {
+                if (seat->priv->active_session == SESSION (greeter))
+                {
+                    l_debug (seat, "Idling greeter");
+                    /* Do this in an idle callback, because we might very well
+                       be in the middle of responding to a START_SESSION
+                       request by a greeter.  So they won't expect an IDLE
+                       call during that.  Plus, this isn't time-sensitive. */
+                    g_idle_add (set_greeter_idle, greeter);
+                }
+            }
+            else
+            {
+                l_debug (seat, "Stopping greeter");
+                session_stop (s);
+            }
         }
     }
 
@@ -260,6 +281,54 @@ seat_get_next_session (Seat *seat)
     return seat->priv->next_session;
 }
 
+/**
+ * Obtains the active session which lightdm expects to be active.
+ *
+ * This function is different from seat_get_active_session() in that the
+ * later (in the case of xlocal seats) dynamically finds the session that is
+ * really active (based on the active VT), whereas this function returns the
+ * session that lightdm activated last by itself, which may not be the actual
+ * active session (i.e. VT changes).
+ */
+Session *
+seat_get_expected_active_session (Seat *seat)
+{
+    g_return_val_if_fail (seat != NULL, NULL);
+    return seat->priv->active_session;
+}
+
+/**
+ * Sets the active session which lightdm expects to be active.
+ *
+ * This function is different from seat_set_active_session() in that the
+ * later performs an actual session activation, whereas this function just
+ * updates the active session after the session has been activated by some
+ * means external to lightdm (i.e. VT changes).
+ */
+void
+seat_set_externally_activated_session (Seat *seat, Session *session)
+{
+    g_return_if_fail (seat != NULL);
+    if (seat->priv->active_session)
+        g_object_unref (seat->priv->active_session);
+    seat->priv->active_session = g_object_ref (session);
+}
+
+Session *
+seat_find_session_by_login1_id (Seat *seat, const gchar *login1_session_id)
+{
+    GList *session_link;
+
+    for (session_link = seat->priv->sessions; session_link; session_link = session_link->next)
+    {
+        Session *session = session_link->data;
+        if (g_strcmp0 (login1_session_id, session_get_login1_session_id (session)) == 0)
+            return session;
+    }
+
+    return NULL;
+}
+
 gboolean
 seat_get_can_switch (Seat *seat)
 {
@@ -279,7 +348,7 @@ run_script (Seat *seat, DisplayServer *display_server, const gchar *script_name,
 {
     Process *script;
     gboolean result = FALSE;
-  
+
     script = process_new (NULL, NULL);
 
     process_set_command (script, script_name);
@@ -329,7 +398,7 @@ run_script (Seat *seat, DisplayServer *display_server, const gchar *script_name,
 
 static void
 seat_real_run_script (Seat *seat, DisplayServer *display_server, Process *process)
-{  
+{
 }
 
 static void
@@ -337,13 +406,15 @@ emit_upstart_signal (const gchar *signal)
 {
     g_return_if_fail (signal != NULL);
     g_return_if_fail (signal[0] != 0);
+    const gchar* argv[] = {"initctl", "-q", "emit", signal, "DISPLAY_MANAGER=lightdm", NULL};
 
     if (getuid () != 0)
         return;
 
-    gchar *cmd = g_strdup_printf ("initctl -q emit %s DISPLAY_MANAGER=lightdm", signal);
-    g_spawn_command_line_async (cmd, NULL); /* OK if it fails, probably not installed */
-    g_free (cmd);
+    /* OK if it fails, probably not installed or not running upstart */
+    g_spawn_async (NULL, argv, NULL,
+            G_SPAWN_SEARCH_PATH | G_SPAWN_STDERR_TO_DEV_NULL,
+            NULL, NULL, NULL, NULL);
 }
 
 static void
@@ -433,36 +504,100 @@ can_share_display_server (Seat *seat, DisplayServer *display_server)
     return seat->priv->share_display_server && display_server_get_can_share (display_server);
 }
 
+static Greeter *
+find_greeter_session (Seat *seat)
+{
+    GList *link;
+
+    for (link = seat->priv->sessions; link; link = link->next)
+    {
+        Session *session = link->data;
+        if (!session_get_is_stopping (session) && IS_GREETER (session))
+            return GREETER (session);
+    }
+
+    return NULL;
+}
+
+static Greeter *
+find_resettable_greeter (Seat *seat)
+{
+    GList *link;
+
+    for (link = seat->priv->sessions; link; link = link->next)
+    {
+        Session *session = link->data;
+        if (!session_get_is_stopping (session) && IS_GREETER (session) &&
+            greeter_get_resettable (GREETER (session)))
+            return GREETER (session);
+    }
+
+    return NULL;
+}
+
+static void
+set_greeter_hints (Seat *seat, Greeter *greeter_session)
+{
+    greeter_clear_hints (greeter_session);
+    greeter_set_hint (greeter_session, "default-session", seat_get_string_property (seat, "user-session"));
+    greeter_set_hint (greeter_session, "hide-users", seat_get_boolean_property (seat, "greeter-hide-users") ? "true" : "false");
+    greeter_set_hint (greeter_session, "show-manual-login", seat_get_boolean_property (seat, "greeter-show-manual-login") ? "true" : "false");
+    greeter_set_hint (greeter_session, "show-remote-login", seat_get_boolean_property (seat, "greeter-show-remote-login") ? "true" : "false");
+    greeter_set_hint (greeter_session, "has-guest-account", seat_get_allow_guest (seat) && seat_get_boolean_property (seat, "greeter-allow-guest") ? "true" : "false");
+}
+
 static void
 switch_to_greeter_from_failed_session (Seat *seat, Session *session)
 {
     Greeter *greeter_session;
+    gboolean existing = FALSE;
+
+    /* Switch to greeter if one open */
+    greeter_session = find_resettable_greeter (seat);
+    if (greeter_session)
+    {
+        l_debug (seat, "Switching to existing greeter");
+        set_greeter_hints (seat, greeter_session);
+        existing = TRUE;
+    }
+    else
+    {
+        greeter_session = create_greeter_session (seat);
+    }
 
-    greeter_session = create_greeter_session (seat);
     if (session_get_is_guest (session))
         greeter_set_hint (greeter_session, "select-guest", "true");
     else
         greeter_set_hint (greeter_session, "select-user", session_get_username (session));
-    if (seat->priv->session_to_activate)
-        g_object_unref (seat->priv->session_to_activate);
-    seat->priv->session_to_activate = g_object_ref (greeter_session);
 
-    if (can_share_display_server (seat, session_get_display_server (session)))
-        session_set_display_server (SESSION (greeter_session), session_get_display_server (session));
+    if (existing)
+    {
+        greeter_reset (greeter_session);
+        seat_set_active_session (seat, SESSION (greeter_session));
+    }
     else
     {
-        DisplayServer *display_server;
+        if (seat->priv->session_to_activate)
+            g_object_unref (seat->priv->session_to_activate);
+        seat->priv->session_to_activate = g_object_ref (greeter_session);
 
-        display_server = create_display_server (seat, session);
-        session_set_display_server (session, display_server);
-        if (!display_server_start (display_server))
+        if (can_share_display_server (seat, session_get_display_server (session)))
+            session_set_display_server (SESSION (greeter_session), session_get_display_server (session));
+        else
         {
-            l_debug (seat, "Failed to start display server for greeter");
-            seat_stop (seat);
-        }
-    }
+            DisplayServer *display_server;
 
-    start_session (seat, SESSION (greeter_session));
+            display_server = create_display_server (seat, session);
+            session_set_display_server (session, display_server);
+            if (!display_server_start (display_server))
+            {
+                l_debug (seat, "Failed to start display server for greeter");
+                seat_stop (seat);
+            }
+        }
+
+        start_session (seat, SESSION (greeter_session));
+    }
 
     /* Stop failed session */
     session_stop (session);
@@ -780,8 +915,8 @@ create_session (Seat *seat, gboolean autostart)
     session = SEAT_GET_CLASS (seat)->create_session (seat);
     seat->priv->sessions = g_list_append (seat->priv->sessions, session);
     if (autostart)
-        g_signal_connect (session, "authentication-complete", G_CALLBACK (session_authentication_complete_cb), seat);
-    g_signal_connect (session, "stopped", G_CALLBACK (session_stopped_cb), seat);
+        g_signal_connect (session, SESSION_SIGNAL_AUTHENTICATION_COMPLETE, G_CALLBACK (session_authentication_complete_cb), seat);
+    g_signal_connect (session, SESSION_SIGNAL_STOPPED, G_CALLBACK (session_stopped_cb), seat);
 
     set_session_env (session);
 
@@ -822,7 +957,7 @@ get_session_argv (Seat *seat, SessionConfig *session_config, const gchar *sessio
         g_free (argv[0]);
         argv[0] = path;
     }
-  
+
     return argv;
 }
 
@@ -865,6 +1000,7 @@ configure_session (Session *session, SessionConfig *config, const gchar *session
     gchar **desktop_names;
 
     session_set_config (session, config);
+    session_set_env (session, "XDG_SESSION_DESKTOP", session_name);
     session_set_env (session, "DESKTOP_SESSION", session_name);
     session_set_env (session, "GDMSESSION", session_name);
     desktop_names = session_config_get_desktop_names (config);
@@ -984,7 +1120,7 @@ create_guest_session (Seat *seat, const gchar *session_name)
     session_set_argv (session, argv);
     g_strfreev (argv);
     g_object_unref (session_config);
-  
+
     return session;
 }
 
@@ -1024,7 +1160,7 @@ greeter_start_session_cb (Greeter *greeter, SessionType type, const gchar *sessi
         gchar **argv;
 
         session = greeter_get_authentication_session (greeter);
-  
+
         /* Get session command to run */
         switch (type)
         {
@@ -1084,7 +1220,8 @@ greeter_start_session_cb (Greeter *greeter, SessionType type, const gchar *sessi
 
     /* If can re-use the display server, stop the greeter first */
     display_server = session_get_display_server (SESSION (greeter));
-    if (can_share_display_server (seat, display_server) &&
+    if (!greeter_get_resettable (greeter) &&
+        can_share_display_server (seat, display_server) &&
         strcmp (display_server_get_session_type (display_server), session_get_session_type (session)) == 0)
     {
         l_debug (seat, "Stopping greeter; display server will be re-used for user session");
@@ -1144,17 +1281,17 @@ create_greeter_session (Seat *seat)
     greeter_session = SEAT_GET_CLASS (seat)->create_greeter_session (seat);
     session_set_config (SESSION (greeter_session), session_config);
     seat->priv->sessions = g_list_append (seat->priv->sessions, SESSION (greeter_session));
-    g_signal_connect (greeter_session, "notify::active-username", G_CALLBACK (greeter_active_username_changed_cb), seat);
-    g_signal_connect (greeter_session, "authentication-complete", G_CALLBACK (session_authentication_complete_cb), seat);
-    g_signal_connect (greeter_session, "stopped", G_CALLBACK (session_stopped_cb), seat);
-  
+    g_signal_connect (greeter_session, GREETER_SIGNAL_ACTIVE_USERNAME_CHANGED, G_CALLBACK (greeter_active_username_changed_cb), seat);
+    g_signal_connect (greeter_session, SESSION_SIGNAL_AUTHENTICATION_COMPLETE, G_CALLBACK (session_authentication_complete_cb), seat);
+    g_signal_connect (greeter_session, SESSION_SIGNAL_STOPPED, G_CALLBACK (session_stopped_cb), seat);
+
     set_session_env (SESSION (greeter_session));
     session_set_env (SESSION (greeter_session), "XDG_SESSION_CLASS", "greeter");
 
     session_set_pam_service (SESSION (greeter_session), seat_get_string_property (seat, "pam-greeter-service"));
     if (getuid () == 0)
     {
-        gchar *greeter_user;      
+        gchar *greeter_user;
         greeter_user = config_get_string (config_get_instance (), "LightDM", "greeter-user");
         session_set_username (SESSION (greeter_session), greeter_user);
         g_free (greeter_user);
@@ -1170,16 +1307,12 @@ create_greeter_session (Seat *seat)
     greeter_set_pam_services (greeter_session,
                               seat_get_string_property (seat, "pam-service"),
                               seat_get_string_property (seat, "pam-autologin-service"));
-    g_signal_connect (greeter_session, "create-session", G_CALLBACK (greeter_create_session_cb), seat);
-    g_signal_connect (greeter_session, "start-session", G_CALLBACK (greeter_start_session_cb), seat);
+    g_signal_connect (greeter_session, GREETER_SIGNAL_CREATE_SESSION, G_CALLBACK (greeter_create_session_cb), seat);
+    g_signal_connect (greeter_session, GREETER_SIGNAL_START_SESSION, G_CALLBACK (greeter_start_session_cb), seat);
 
     /* Set hints to greeter */
-    greeter_set_hint (greeter_session, "default-session", seat_get_string_property (seat, "user-session"));
     greeter_set_allow_guest (greeter_session, seat_get_allow_guest (seat));
-    greeter_set_hint (greeter_session, "hide-users", seat_get_boolean_property (seat, "greeter-hide-users") ? "true" : "false");
-    greeter_set_hint (greeter_session, "show-manual-login", seat_get_boolean_property (seat, "greeter-show-manual-login") ? "true" : "false");
-    greeter_set_hint (greeter_session, "show-remote-login", seat_get_boolean_property (seat, "greeter-show-remote-login") ? "true" : "false");
-    greeter_set_hint (greeter_session, "has-guest-account", seat_get_allow_guest (seat) && seat_get_boolean_property (seat, "greeter-allow-guest") ? "true" : "false");
+    set_greeter_hints (seat, greeter_session);
 
     /* Configure for automatic login */
     autologin_username = seat_get_string_property (seat, "autologin-user");
@@ -1271,27 +1404,12 @@ create_display_server (Seat *seat, Session *session)
         return NULL;
 
     seat->priv->display_servers = g_list_append (seat->priv->display_servers, display_server);
-    g_signal_connect (display_server, "ready", G_CALLBACK (display_server_ready_cb), seat);
-    g_signal_connect (display_server, "stopped", G_CALLBACK (display_server_stopped_cb), seat);
+    g_signal_connect (display_server, DISPLAY_SERVER_SIGNAL_READY, G_CALLBACK (display_server_ready_cb), seat);
+    g_signal_connect (display_server, DISPLAY_SERVER_SIGNAL_STOPPED, G_CALLBACK (display_server_stopped_cb), seat);
 
     return display_server;
 }
 
-static Greeter *
-find_greeter_session (Seat *seat)
-{
-    GList *link;
-
-    for (link = seat->priv->sessions; link; link = link->next)
-    {
-        Session *session = link->data;
-        if (!session_get_is_stopping (session) && IS_GREETER (session))
-            return GREETER (session);
-    }
-
-    return NULL;
-}
-
 gboolean
 seat_switch_to_greeter (Seat *seat)
 {
@@ -1303,7 +1421,7 @@ seat_switch_to_greeter (Seat *seat)
     if (!seat_get_can_switch (seat))
         return FALSE;
 
-    /* Switch to greeter if one open (shouldn't be though) */
+    /* Switch to greeter if one open */
     greeter_session = find_greeter_session (seat);
     if (greeter_session)
     {
@@ -1322,10 +1440,8 @@ seat_switch_to_greeter (Seat *seat)
 
     display_server = create_display_server (seat, SESSION (greeter_session));
     session_set_display_server (SESSION (greeter_session), display_server);
-    if (!display_server_start (display_server))
-        return FALSE;
 
-    return TRUE;
+    return display_server_start (display_server);
 }
 
 static void
@@ -1333,6 +1449,7 @@ switch_authentication_complete_cb (Session *session, Seat *seat)
 {
     Greeter *greeter_session;
     DisplayServer *display_server;
+    gboolean existing = FALSE;
 
     /* If authenticated, then unlock existing session or start new one */
     if (session_get_is_authenticated (session))
@@ -1361,22 +1478,42 @@ switch_authentication_complete_cb (Session *session, Seat *seat)
         return;
     }
 
-    l_debug (seat, "Switching to greeter to authenticate session");
-
     session_stop (session);
 
-    greeter_session = create_greeter_session (seat);
+    /* See if we already have a greeter up and reuse it if so */
+    greeter_session = find_resettable_greeter (seat);
+    if (greeter_session)
+    {
+        l_debug (seat, "Switching to existing greeter to authenticate session");
+        set_greeter_hints (seat, greeter_session);
+        existing = TRUE;
+    }
+    else
+    {
+        l_debug (seat, "Starting greeter to authenticate session");
+        greeter_session = create_greeter_session (seat);
+    }
+
     if (session_get_is_guest (session))
         greeter_set_hint (greeter_session, "select-guest", "true");
     else
         greeter_set_hint (greeter_session, "select-user", session_get_username (session));
-    if (seat->priv->session_to_activate)
-        g_object_unref (seat->priv->session_to_activate);
-    seat->priv->session_to_activate = g_object_ref (greeter_session);
 
-    display_server = create_display_server (seat, SESSION (greeter_session));
-    session_set_display_server (SESSION (greeter_session), display_server);
-    display_server_start (display_server);
+    if (existing)
+    {
+        greeter_reset (greeter_session);
+        seat_set_active_session (seat, SESSION (greeter_session));
+    }
+    else
+    {
+        if (seat->priv->session_to_activate)
+            g_object_unref (seat->priv->session_to_activate);
+        seat->priv->session_to_activate = g_object_ref (greeter_session);
+
+        display_server = create_display_server (seat, SESSION (greeter_session));
+        session_set_display_server (SESSION (greeter_session), display_server);
+        display_server_start (display_server);
+    }
 }
 
 gboolean
@@ -1399,7 +1536,7 @@ seat_switch_to_user (Seat *seat, const gchar *username, const gchar *session_nam
 
     /* Attempt to authenticate them */
     session = create_user_session (seat, username, FALSE);
-    g_signal_connect (session, "authentication-complete", G_CALLBACK (switch_authentication_complete_cb), seat);
+    g_signal_connect (session, SESSION_SIGNAL_AUTHENTICATION_COMPLETE, G_CALLBACK (switch_authentication_complete_cb), seat);
     session_set_pam_service (session, seat_get_string_property (seat, "pam-service"));
 
     return session_start (session);
@@ -1460,6 +1597,7 @@ seat_lock (Seat *seat, const gchar *username)
 {
     Greeter *greeter_session;
     DisplayServer *display_server;
+    gboolean existing = FALSE;
 
     g_return_val_if_fail (seat != NULL, FALSE);
 
@@ -1468,30 +1606,42 @@ seat_lock (Seat *seat, const gchar *username)
 
     l_debug (seat, "Locking");
 
-    /* Switch to greeter if one open (shouldn't be though) */
-    greeter_session = find_greeter_session (seat);
+    /* Switch to greeter we can reuse */
+    greeter_session = find_resettable_greeter (seat);
     if (greeter_session)
     {
         l_debug (seat, "Switching to existing greeter");
-        seat_set_active_session (seat, SESSION (greeter_session));
-        return TRUE;
+        set_greeter_hints (seat, greeter_session);
+        existing = TRUE;
+    }
+    else
+    {
+        greeter_session = create_greeter_session (seat);
+        if (!greeter_session)
+            return FALSE;
     }
 
-    greeter_session = create_greeter_session (seat);
-    if (!greeter_session)
-        return FALSE;
-
-    display_server = create_display_server (seat, SESSION (greeter_session));
-
-    if (seat->priv->session_to_activate)
-        g_object_unref (seat->priv->session_to_activate);
-    seat->priv->session_to_activate = g_object_ref (greeter_session);
     greeter_set_hint (greeter_session, "lock-screen", "true");
     if (username)
         greeter_set_hint (greeter_session, "select-user", username);
-    session_set_display_server (SESSION (greeter_session), display_server);
 
-    return display_server_start (display_server);
+    if (existing)
+    {
+        greeter_reset (greeter_session);
+        seat_set_active_session (seat, SESSION (greeter_session));
+        return TRUE;
+    }
+    else
+    {
+        display_server = create_display_server (seat, SESSION (greeter_session));
+
+        if (seat->priv->session_to_activate)
+            g_object_unref (seat->priv->session_to_activate);
+        seat->priv->session_to_activate = g_object_ref (greeter_session);
+        session_set_display_server (SESSION (greeter_session), display_server);
+
+        return display_server_start (display_server);
+    }
 }
 
 void
@@ -1553,7 +1703,7 @@ seat_real_start (Seat *seat)
             background_session = session;
             session = NULL;
         }
-      
+
         if (session)
         {
             DisplayServer *display_server;
@@ -1593,19 +1743,6 @@ seat_real_start (Seat *seat)
         seat->priv->session_to_activate = g_object_ref (greeter_session);
         session = SESSION (greeter_session);
 
-        if (autologin_timeout)
-        {
-            gchar *value;
-
-            value = g_strdup_printf ("%d", autologin_timeout);
-            greeter_set_hint (greeter_session, "autologin-timeout", value);
-            g_free (value);
-            if (autologin_username)
-                greeter_set_hint (greeter_session, "autologin-user", autologin_username);
-            if (autologin_guest)
-                greeter_set_hint (greeter_session, "autologin-guest", "true");
-        }
-
         display_server = create_display_server (seat, session);
         session_set_display_server (session, display_server);
         if (!display_server || !display_server_start (display_server))
@@ -1722,12 +1859,13 @@ seat_finalize (GObject *object)
 
     self = SEAT (object);
 
+    g_free (self->priv->name);
     g_hash_table_unref (self->priv->properties);
     for (link = self->priv->display_servers; link; link = link->next)
     {
         DisplayServer *display_server = link->data;
         g_signal_handlers_disconnect_matched (display_server, G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, self);
-    }  
+    }
     g_list_free_full (self->priv->display_servers, g_object_unref);
     for (link = self->priv->sessions; link; link = link->next)
     {
@@ -1765,7 +1903,7 @@ seat_class_init (SeatClass *klass)
     g_type_class_add_private (klass, sizeof (SeatPrivate));
 
     signals[SESSION_ADDED] =
-        g_signal_new ("session-added",
+        g_signal_new (SEAT_SIGNAL_SESSION_ADDED,
                       G_TYPE_FROM_CLASS (klass),
                       G_SIGNAL_RUN_LAST,
                       G_STRUCT_OFFSET (SeatClass, session_added),
@@ -1773,7 +1911,7 @@ seat_class_init (SeatClass *klass)
                       NULL,
                       G_TYPE_NONE, 1, SESSION_TYPE);
     signals[RUNNING_USER_SESSION] =
-        g_signal_new ("running-user-session",
+        g_signal_new (SEAT_SIGNAL_RUNNING_USER_SESSION,
                       G_TYPE_FROM_CLASS (klass),
                       G_SIGNAL_RUN_LAST,
                       G_STRUCT_OFFSET (SeatClass, running_user_session),
@@ -1781,7 +1919,7 @@ seat_class_init (SeatClass *klass)
                       NULL,
                       G_TYPE_NONE, 1, SESSION_TYPE);
     signals[SESSION_REMOVED] =
-        g_signal_new ("session-removed",
+        g_signal_new (SEAT_SIGNAL_SESSION_REMOVED,
                       G_TYPE_FROM_CLASS (klass),
                       G_SIGNAL_RUN_LAST,
                       G_STRUCT_OFFSET (SeatClass, session_removed),
@@ -1789,7 +1927,7 @@ seat_class_init (SeatClass *klass)
                       NULL,
                       G_TYPE_NONE, 1, SESSION_TYPE);
     signals[STOPPED] =
-        g_signal_new ("stopped",
+        g_signal_new (SEAT_SIGNAL_STOPPED,
                       G_TYPE_FROM_CLASS (klass),
                       G_SIGNAL_RUN_LAST,
                       G_STRUCT_OFFSET (SeatClass, stopped),
@@ -1801,12 +1939,7 @@ seat_class_init (SeatClass *klass)
 static gint
 seat_real_logprefix (Logger *self, gchar *buf, gulong buflen)
 {
-    Seat *seat = SEAT (self);
-    const gchar *name = seat_get_string_property (seat, "seat-name");
-    if (name)
-        return g_snprintf (buf, buflen, "Seat %s: ", name);
-    else
-        return g_snprintf (buf, buflen, "Seat: ");
+    return g_snprintf (buf, buflen, "Seat %s: ", SEAT (self)->priv->name);
 }
 
 static void
