@@ -72,7 +72,7 @@ read_data (void *buf, size_t count)
     n_read = read (from_daemon_output, buf, count);
     if (n_read < 0)
         g_printerr ("Error reading from daemon: %s\n", strerror (errno));
-  
+
     return n_read;
 }
 
@@ -91,10 +91,10 @@ read_string_full (void* (*alloc_fn)(size_t n))
         g_printerr ("Invalid string length %d from daemon\n", length);
         return NULL;
     }
-  
+
     value = (*alloc_fn) (sizeof (char) * (length + 1));
     read_data (value, length);
-    value[length] = '\0';      
+    value[length] = '\0';
 
     return value;
 }
@@ -221,7 +221,7 @@ updwtmpx (const gchar *wtmp_file, struct utmpx *ut)
         strncpy (u.ut_host, ut->ut_host, sizeof (u.ut_host));
     u.ut_tv.tv_sec = ut->ut_tv.tv_sec;
     u.ut_tv.tv_usec = ut->ut_tv.tv_usec;
-  
+
     updwtmp (wtmp_file, &u);
 }
 
@@ -275,7 +275,7 @@ session_child_run (int argc, char **argv)
     XAuthority *x_authority = NULL;
     gchar *x_authority_filename;
     GDBusConnection *bus;
-    const gchar *login1_session = NULL;
+    const gchar *login1_session_id = NULL;
     gchar *console_kit_cookie = NULL;
     const gchar *locale_value;
     gchar *locale_var;
@@ -367,7 +367,7 @@ session_child_run (int argc, char **argv)
         pam_set_item (pam_handle, PAM_TTY, xdisplay);
     }
     else if (tty)
-        pam_set_item (pam_handle, PAM_TTY, tty);    
+        pam_set_item (pam_handle, PAM_TTY, tty);
 
 #ifdef PAM_XAUTHDATA
     if (x_authority)
@@ -501,7 +501,7 @@ session_child_run (int argc, char **argv)
     if (version >= 1)
     {
         g_free (tty);
-        tty = read_string ();      
+        tty = read_string ();
     }
     x_authority_filename = read_string ();
     if (version >= 1)
@@ -543,7 +543,7 @@ session_child_run (int argc, char **argv)
     }
     else
     {
-        fd = open ("/dev/null", O_WRONLY);   
+        fd = open ("/dev/null", O_WRONLY);
         dup2 (fd, STDERR_FILENO);
         close (fd);
     }
@@ -566,7 +566,7 @@ session_child_run (int argc, char **argv)
         pam_end (pam_handle, 0);
         return EXIT_FAILURE;
     }
-     
+
     /* Open the session */
     result = pam_open_session (pam_handle, 0);
     if (result != PAM_SUCCESS)
@@ -586,16 +586,16 @@ session_child_run (int argc, char **argv)
         return EXIT_FAILURE;
     }
 
-    login1_session = pam_getenv (pam_handle, "XDG_SESSION_ID");
-    if (login1_session)
+    /* Check what logind session we are, or fallback to ConsoleKit */
+    login1_session_id = pam_getenv (pam_handle, "XDG_SESSION_ID");
+    if (login1_session_id)
     {
-        write_string (login1_session);
+        write_string (login1_session_id);
         if (version >= 2)
             write_string (NULL);
     }
     else
     {
-        /* Open a Console Kit session */
         g_variant_builder_init (&ck_parameters, G_VARIANT_TYPE ("(a(sv))"));
         g_variant_builder_open (&ck_parameters, G_VARIANT_TYPE ("a(sv)"));
         g_variant_builder_add (&ck_parameters, "(sv)", "unix-user", g_variant_new_int32 (user_get_uid (user)));
@@ -817,7 +817,7 @@ session_child_run (int argc, char **argv)
     pam_close_session (pam_handle, 0);
 
     /* Remove credentials */
-    result = pam_setcred (pam_handle, PAM_DELETE_CRED);
+    pam_setcred (pam_handle, PAM_DELETE_CRED);
 
     pam_end (pam_handle, 0);
     pam_handle = NULL;
